@@ -23,50 +23,27 @@ SOFTWARE IS DISCLAIMED.
 #ifndef KADDRBOOK_H
 #define KADDRBOOK_H
 
-#include <kabc/stdaddressbook.h>
-#include <kabc/addressbook.h>
-#include <kabc/vcardconverter.h>
 #include <kabc/resource.h>
 
-#include "osyncbase.h"
+#include "datasource.h"
 
-namespace KABC
+class KContactDataSource : public OSyncDataSource
 {
-class Addressee;
-class Address;
-class Ticket;
-}
-
-using namespace KABC;
-
-class KContactDataSource
-{
-	private:
-		AddressBook* addressbookptr;
-		// for a local file, we need to obtain a saveTicket when opening the abook
-		Ticket*ticket;
-		//help to only save when when changed
-		bool abChanged;
-
-		KABC::VCardConverter* converter;
-
-		OSyncHashTable *hashtable;
-		OSyncMember *member;
-
-		bool __vcard_access(OSyncContext *ctx, OSyncChange *chg);
-		OSyncChange *_addressee_to_change(Addressee *a);
-
 	public:
-		KContactDataSource(OSyncMember *member, OSyncHashTable *hashtable);
-		~KContactDataSource();
+		KContactDataSource() : OSyncDataSource("contact"), addressbookptr(0), modified(false), ticket(0) {};
+		virtual ~KContactDataSource() {};
 
-		QString calc_hash(Addressee &e);
-		bool connect(OSyncContext *ctx);
-		bool disconnect(OSyncContext *ctx);
-		bool contact_get_changeinfo(OSyncContext *ctx);
-		bool vcard_access(OSyncContext *ctx, OSyncChange *chg);
-		bool vcard_commit_change(OSyncContext *ctx, OSyncChange *chg);
-		bool connected;
+		virtual void connect(OSyncPluginInfo *info, OSyncContext *ctx);
+		virtual void disconnect(OSyncPluginInfo *info, OSyncContext *ctx);
+		virtual void get_changes(OSyncPluginInfo *info, OSyncContext *ctx);
+		virtual void commit(OSyncPluginInfo *info, OSyncContext *ctx, OSyncChange *chg);
+
+	private:
+		QString calc_hash(KABC::Addressee &e);
+
+		KABC::AddressBook* addressbookptr;
+		bool modified;  // set when needed to save addressbook back
+		KABC::Ticket *ticket;
 };
 
 #endif
