@@ -20,22 +20,23 @@ COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS, RELATING TO USE OF THIS
 SOFTWARE IS DISCLAIMED.
 *************************************************************************/
 /**
- * @autor Eduardo Pereira Habkost <ehabkost@conectiva.com.br>
+ * @author Eduardo Pereira Habkost <ehabkost@conectiva.com.br>
+ * @author Andrew Baumann <andrewb@cse.unsw.edu.au>
  */
 
 #include "kcal.h"
 
+#include <assert.h>
 #include <kapplication.h>
 #include <dcopclient.h>
 
-KCalDataSource::KCalDataSource(OSyncMember *member, OSyncHashTable *hashtable)
-		: hashtable(hashtable), member(member)
+bool KCalSharedResource::open(OSyncContext *ctx)
 {
-	connected = false;
-}
+	if (refcount++ > 0) {
+		assert(calendar);
+		return true;
+	}
 
-bool KCalDataSource::connect(OSyncContext *ctx)
-{
 	DCOPClient *dcopc = KApplication::kApplication()->dcopClient();
 	if (!dcopc) {
 		osync_context_report_error(ctx, OSYNC_ERROR_INITIALIZATION, "Unable to initialize dcop client");
@@ -66,36 +67,24 @@ bool KCalDataSource::connect(OSyncContext *ctx)
 	calendar->load();
 #endif
 
-	//Detection mechanismn if this is the first sync
-	if (osync_member_objtype_enabled(member, "event")
-	    && !osync_anchor_compare(member, "event", "true")) {
-		osync_trace(TRACE_INTERNAL, "Setting slow-sync for event");
-		osync_member_set_slow_sync(member, "event", TRUE);
-	}
+//	osync_debug("kcal", 3, "Calendar: %d events", calendar->events().size());
 
-	if (osync_member_objtype_enabled(member, "todo")
-	    && !osync_anchor_compare(member, "todo", "true")) {
-		osync_trace(TRACE_INTERNAL, "Setting slow-sync for todo");
-		osync_member_set_slow_sync(member, "todo", TRUE);
-	}
-
-	osync_debug("kcal", 3, "Calendar: %d events", calendar->events().size());
-
-	connected = true;
 	return true;
 }
 
-bool KCalDataSource::disconnect(OSyncContext *)
+bool KCalSharedResource::close(OSyncContext *)
 {
 	/* Save the changes */
 	calendar->save();
+	
+	if (--refcount > 0)
+		return true;
 
 	delete calendar;
 	calendar = NULL;
-	connected = false;
 	return true;
 }
-
+	
 static QString calc_hash(const KCal::Incidence *e)
 {
 	QDateTime d = e->lastModified();
@@ -107,115 +96,15 @@ static QString calc_hash(const KCal::Incidence *e)
 	return d.toString();
 }
 
-/** Report a list of calendar incidences (events or to-dos), with the
- * right objtype and objformat.
- *
- * This function exists because the logic for converting the events or to-dos
- * is the same, only the objtype and format is different.
- */
-bool KCalDataSource::report_incidence(OSyncContext *ctx, KCal::Incidence *e, const char *objtype, const char *objformat)
-{
-	osync_debug("kcal", 3, "One calendar incidence (%s)", objtype);
-	QString hash = calc_hash(e);
-
-	QString uid = e->uid();
-
-	/* Build a local calendar for the incidence data */
-	KCal::CalendarLocal cal(calendar->timeZoneId());
-	osync_debug("kcal", 3, "timezoneid: %s\n", (const char*)cal.timeZoneId().local8Bit());
-	cal.addIncidence(e->clone());
-
-	/* Convert the data to vcalendar */
-	KCal::ICalFormat format;
-	QCString datastr = format.toString(&cal).utf8();
-	const char *data = datastr;
-
-	osync_debug("kcal", 3, "UID: %s\n", (const char*)uid.local8Bit());
-	OSyncChange *chg = osync_change_new();
-	osync_change_set_uid(chg, uid.local8Bit());
-	osync_change_set_member(chg, member);
-
-	// object type and format
-	osync_change_set_objtype_string(chg, objtype);
-	osync_change_set_objformat_string(chg, objformat);
-	osync_change_set_data(chg, strdup(data), strlen(data) +1, 1);
-
-	// Use the hash table to check if the object
-	// needs to be reported
-	osync_change_set_hash(chg, hash.data());
-	if (osync_hashtable_detect_change(hashtable, chg)) {
-		osync_context_report_change(ctx, chg);
-		osync_hashtable_update_hash(hashtable, chg);
-	}
-
-	return true;
-}
-
-bool KCalDataSource::get_changeinfo_events(OSyncContext *ctx)
-{
-	KCal::Event::List events = calendar->events();
-	osync_debug("kcal", 3, "Number of events: %d", events.size());
-
-	if (osync_member_get_slow_sync(member, "event")) {
-		osync_debug("kcal", 3, "Setting slow-sync for events");
-		osync_hashtable_set_slow_sync(hashtable, "event");
-	}
-
-	for (KCal::Event::List::ConstIterator i = events.begin(); i != events.end(); i++) {
-
-		/* Skip entries from birthday resource. This is just a workaround.
-		 * patch by rhuitl
-		 * FIXME: todo: add a list of resources to kdepim-sync.conf
-		 */
-		if ( (*i)->uid().contains("KABC_Birthday") || (*i)->uid().contains("KABC_Anniversary") )
-			continue;
-
-		if (!report_incidence(ctx, *i, "event", "vevent20"))
-			return false;
-	}
-
-	osync_hashtable_report_deleted(hashtable, ctx, "event");
-
-	return true;
-}
-
-bool KCalDataSource::get_changeinfo_todos(OSyncContext *ctx)
-{
-	KCal::Todo::List todos = calendar->todos();
-
-	osync_debug("kcal", 3, "Number of to-dos: %d", todos.size());
-
-	if (osync_member_get_slow_sync(member, "todo")) {
-		osync_debug("kcal", 3, "Setting slow-sync for todos");
-		osync_hashtable_set_slow_sync(hashtable, "todo");
-	}
-
-	for (KCal::Todo::List::ConstIterator i = todos.begin(); i != todos.end(); i++) {
-		osync_debug("kcal", 3, "%p: doesFloat: %d", *i, (*i)->doesFloat());
-		if (!report_incidence(ctx, *i, "todo", "vtodo20"))
-			return false;
-	}
-
-	osync_hashtable_report_deleted(hashtable, ctx, "todo");
-
-	return true;
-
-}
-
-void KCalDataSource::get_data(OSyncContext *ctx, OSyncChange *)
-{
-	osync_context_report_error(ctx, OSYNC_ERROR_NOT_SUPPORTED, "Not implemented yet");
-}
-
 /** Add or change a incidence on the calendar. This function
  * is used for events and to-dos
  */
-bool KCalDataSource::__access(OSyncContext *ctx, OSyncChange *chg)
+bool KCalSharedResource::commit(OSyncContext *ctx, OSyncChange *chg)
 {
 	OSyncChangeType type = osync_change_get_changetype(chg);
-	osync_debug("kcal", 3, "%s", __FUNCTION__);
+//	osync_debug("kcal", 3, "%s", __FUNCTION__);
 	switch (type) {
-		case CHANGE_DELETED: {
+		case OSYNC_CHANGE_TYPE_DELETED: {
 			KCal::Incidence *e = calendar->incidence(osync_change_get_uid(chg));
 			if (!e) {
 				osync_context_report_error(ctx, OSYNC_ERROR_FILE_NOT_FOUND, "Event not found while deleting");
@@ -224,28 +113,29 @@ bool KCalDataSource::__access(OSyncContext *ctx, OSyncChange *chg)
 			calendar->deleteIncidence(e);
 			break;
 		}
-		case CHANGE_ADDED:
-		case CHANGE_MODIFIED: {
+		case OSYNC_CHANGE_TYPE_ADDED:
+		case OSYNC_CHANGE_TYPE_MODIFIED: {
 			KCal::ICalFormat format;
+	
+			OSyncData *odata = osync_change_get_data(chg);
+
+			char *databuf;
+			//size_t databuf_size; 
+			// osync_data_get_data requires an unsigned int which is not compatible with size_t on 64bit machines
+			unsigned int databuf_size = 0; 
+			osync_data_get_data(odata, &databuf, &databuf_size);
 
 			/* First, parse to a temporary calendar, because
 				* we should set the uid on the events
 				*/
 
 			KCal::CalendarLocal cal(QString::fromLatin1( "UTC" ));
-			QString data = QString::fromUtf8(osync_change_get_data(chg), osync_change_get_datasize(chg));
+			QString data = QString::fromUtf8(databuf, databuf_size);
 			if (!format.fromString(&cal, data)) {
 				osync_context_report_error(ctx, OSYNC_ERROR_CONVERT, "Couldn't import calendar data");
 				return false;
 			}
 
-			/*FIXME: The event/to-do will be overwritten. But I can't differentiate
-				* between a field being removed and a missing field because
-				* the other device don't support them, because OpenSync currently
-				* doesn't have support for it. Yes, it is risky, but unfortunately
-				* CalendarResources don't have support for changing an event
-				* from vcard data, and not adding it.
-				*/
 			KCal::Incidence *oldevt = calendar->incidence(osync_change_get_uid(chg));
 			if (oldevt)
 				calendar->deleteIncidence(oldevt);
@@ -257,11 +147,11 @@ bool KCalDataSource::__access(OSyncContext *ctx, OSyncChange *chg)
 			KCal::Incidence::List evts = cal.incidences();
 			for (KCal::Incidence::List::ConstIterator i = evts.begin(); i != evts.end(); i++) {
 				KCal::Incidence *e = (*i)->clone();
-				if (type == CHANGE_MODIFIED)
+				if (type == OSYNC_CHANGE_TYPE_MODIFIED)
 					e->setUid(osync_change_get_uid(chg));
 
-				osync_debug("kcal", 3, "Writing incidence: uid: %s, summary: %s",
-						(const char*)e->uid().local8Bit(), (const char*)e->summary().local8Bit());
+/*				osync_debug("kcal", 3, "Writing incidence: uid: %s, summary: %s",
+						(const char*)e->uid().local8Bit(), (const char*)e->summary().local8Bit()); */
 
 				QString c_uid = e->uid().utf8();
 				osync_change_set_uid(chg, (const char*)c_uid);
@@ -280,42 +170,182 @@ bool KCalDataSource::__access(OSyncContext *ctx, OSyncChange *chg)
 	return true;
 }
 
-bool KCalDataSource::event_access(OSyncContext *ctx, OSyncChange *chg)
+/** Report a list of calendar incidences (events or to-dos), with the
+ * right objtype and objformat.
+ *
+ * This function exists because the logic for converting the events or to-dos
+ * is the same, only the objtype and format is different.
+ */
+bool KCalSharedResource::report_incidence(OSyncDataSource *dsobj, OSyncPluginInfo *info, OSyncContext *ctx, KCal::Incidence *e, OSyncObjFormat *objformat)
+{
+	/* Build a local calendar for the incidence data */
+	KCal::CalendarLocal cal(calendar->timeZoneId());
+//	osync_debug("kcal", 3, "timezoneid: %s\n", (const char*)cal.timeZoneId().local8Bit());
+	cal.addIncidence(e->clone());
+
+	/* Convert the data to vcalendar */
+	KCal::ICalFormat format;
+	QString data = format.toString(&cal);
+
+	return dsobj->report_change(info, ctx, e->uid(), data, calc_hash(e), objformat);
+}
+
+bool KCalSharedResource::get_event_changes(OSyncDataSource *dsobj, OSyncPluginInfo *info, OSyncContext *ctx)
+{
+	OSyncFormatEnv *formatenv = osync_plugin_info_get_format_env(info);
+	OSyncObjFormat *objformat = osync_format_env_find_objformat(formatenv, "vevent20");
+	
+	KCal::Event::List events = calendar->events();
+//	osync_debug("kcal", 3, "Number of events: %d", events.size());
+
+	for (KCal::Event::List::ConstIterator i = events.begin(); i != events.end(); i++) {
+
+		/* Skip entries from birthday resource. This is just a workaround.
+		 * patch by rhuitl
+		 * FIXME: todo: add a list of resources to kdepim-sync.conf
+		 */
+		if ( (*i)->uid().contains("KABC_Birthday") || (*i)->uid().contains("KABC_Anniversary") )
+			continue;
+
+		if (!report_incidence(dsobj, info, ctx, *i, objformat))
+			return false;
+	}
+
+	return true;
+}
+
+bool KCalSharedResource::get_todo_changes(OSyncDataSource *dsobj, OSyncPluginInfo *info, OSyncContext *ctx)
+{
+	OSyncFormatEnv *formatenv = osync_plugin_info_get_format_env(info);
+	OSyncObjFormat *objformat = osync_format_env_find_objformat(formatenv, "vtodo20");
+	
+	KCal::Todo::List todos = calendar->todos();
+
+//	osync_debug("kcal", 3, "Number of to-dos: %d", todos.size());
+
+	for (KCal::Todo::List::ConstIterator i = todos.begin(); i != todos.end(); i++) {
+//		osync_debug("kcal", 3, "%p: doesFloat: %d", *i, (*i)->doesFloat());
+		if (!report_incidence(dsobj, info, ctx, *i, objformat))
+			return false;
+	}
+
+	return true;
+}
+
+void KCalEventDataSource::connect(OSyncPluginInfo *info, OSyncContext *ctx)
+{
+	if (!kcal->open(ctx))
+		return;
+	OSyncDataSource::connect(info, ctx);
+}
+
+void KCalTodoDataSource::connect(OSyncPluginInfo *info, OSyncContext *ctx)
+{
+	if (kcal->open(ctx))
+		OSyncDataSource::connect(info, ctx);
+}
+
+void KCalEventDataSource::disconnect(OSyncPluginInfo *, OSyncContext *ctx)
+{
+	if (kcal->close(ctx))
+		osync_context_report_success(ctx);
+}
+
+void KCalTodoDataSource::disconnect(OSyncPluginInfo *, OSyncContext *ctx)
+{
+	if (kcal->close(ctx))
+		osync_context_report_success(ctx);
+}
+
+void KCalEventDataSource::get_changes(OSyncPluginInfo *info, OSyncContext *ctx)
+{
+	osync_trace(TRACE_ENTRY, "%s(%p, %p)", __PRETTY_FUNCTION__, info, ctx);
+
+	OSyncError *error = NULL;
+
+	OSyncObjTypeSink *sink = osync_plugin_info_get_sink(info);
+
+	OSyncFormatEnv *formatenv = osync_plugin_info_get_format_env(info);
+	OSyncObjFormat *objformat = osync_format_env_find_objformat(formatenv, "vevent20");
+
+	if (osync_objtype_sink_get_slowsync(sink)) {
+		osync_trace(TRACE_INTERNAL, "Got slow-sync");
+		if (!osync_hashtable_slowsync(hashtable, &error)) {
+			osync_context_report_osyncerror(ctx, error);
+			osync_trace(TRACE_EXIT_ERROR, "%s: %s", __PRETTY_FUNCTION__, osync_error_print(&error));
+			return;
+		}
+	}
+
+	if (!kcal->get_event_changes(this, info, ctx)) {
+		osync_context_report_error(ctx, OSYNC_ERROR_GENERIC, "Error while reciving latest changes.");
+		osync_trace(TRACE_EXIT_ERROR, "%s: error in get_todo_changes", __PRETTY_FUNCTION__);
+		return;
+	}
+
+	if (!report_deleted(info, ctx, objformat)) {
+		osync_context_report_error(ctx, OSYNC_ERROR_GENERIC, "Error while detecting latest changes.");
+		osync_trace(TRACE_EXIT_ERROR, "%s", __PRETTY_FUNCTION__);
+		return;
+	}
+
+	osync_context_report_success(ctx);
+	osync_trace(TRACE_EXIT, "%s", __PRETTY_FUNCTION__);
+}
+
+void KCalTodoDataSource::get_changes(OSyncPluginInfo *info, OSyncContext *ctx)
+{
+	osync_trace(TRACE_ENTRY, "%s(%p, %p)", __PRETTY_FUNCTION__, info, ctx);
+
+	OSyncError *error = NULL;
+
+	OSyncObjTypeSink *sink = osync_plugin_info_get_sink(info);
+
+	OSyncFormatEnv *formatenv = osync_plugin_info_get_format_env(info);
+	OSyncObjFormat *objformat = osync_format_env_find_objformat(formatenv, "vtodo20");
+
+	if (osync_objtype_sink_get_slowsync(sink)) {
+		osync_trace(TRACE_INTERNAL, "Got slow-sync");
+		if (!osync_hashtable_slowsync(hashtable, &error)) {
+			osync_context_report_osyncerror(ctx, error);
+			osync_trace(TRACE_EXIT_ERROR, "%s: %s", __PRETTY_FUNCTION__, osync_error_print(&error));
+			return;
+		}
+
+	}
+
+	if (!kcal->get_todo_changes(this, info, ctx)) {
+		osync_trace(TRACE_EXIT_ERROR, "%s: error in get_todo_changes", __PRETTY_FUNCTION__);
+		osync_context_report_error(ctx, OSYNC_ERROR_GENERIC, "Error while detecting latest changes.");
+		return;
+	}
+
+	if (!report_deleted(info, ctx, objformat)) {
+		osync_trace(TRACE_EXIT_ERROR, "%s", __PRETTY_FUNCTION__);
+		osync_context_report_error(ctx, OSYNC_ERROR_GENERIC, "Error while detecting deleted entries.");
+		return;
+	}
+
+	osync_context_report_success(ctx);
+	osync_trace(TRACE_EXIT, "%s", __PRETTY_FUNCTION__);
+}
+
+void KCalEventDataSource::commit(OSyncPluginInfo *, OSyncContext *ctx, OSyncChange *chg)
 {
 	// We use the same function for events and to-do
-	if (!__access(ctx, chg))
-		return false;
-
+	if (!kcal->commit(ctx, chg))
+		return;
+	
+	osync_hashtable_update_change(hashtable, chg);
 	osync_context_report_success(ctx);
-	return true;
 }
 
-bool KCalDataSource::event_commit_change(OSyncContext *ctx, OSyncChange *chg)
-{
-	// We use the same function for events and to-do
-	if ( !__access(ctx, chg) )
-		return false;
-	osync_hashtable_update_hash(hashtable, chg);
-	osync_context_report_success(ctx);
-	return true;
-}
-
-bool KCalDataSource::todo_access(OSyncContext *ctx, OSyncChange *chg)
+void KCalTodoDataSource::commit(OSyncPluginInfo *, OSyncContext *ctx, OSyncChange *chg)
 {
 	// We use the same function for calendar and to-do
-	if (!__access(ctx, chg))
-		return false;
-
+	if (!kcal->commit(ctx, chg))
+		return;
+	
+	osync_hashtable_update_change(hashtable, chg);
 	osync_context_report_success(ctx);
-	return true;
-}
-
-bool KCalDataSource::todo_commit_change(OSyncContext *ctx, OSyncChange *chg)
-{
-	// We use the same function for calendar and to-do
-	if ( !__access(ctx, chg) )
-		return false;
-	osync_hashtable_update_hash(hashtable, chg);
-	osync_context_report_success(ctx);
-	return true;
 }
