@@ -21,36 +21,31 @@ SOFTWARE IS DISCLAIMED.
 *************************************************************************/
 /** @file
  *
- * @autor Eduardo Pereira Habkost <ehabkost@conectiva.com.br>
+ * @author Eduardo Pereira Habkost <ehabkost@conectiva.com.br>
+ * @author Andrew Baumann <andrewb@cse.unsw.edu.au>
  *
  * This module implements the access to the KDE 3.2 Notes, that are
  * stored on KGlobal::dirs()->saveLocation( "data" , "knotes/" ) + "notes.ics"
  *
  */
 
-
-
 #include "knotes.h"
-#include <glib.h>
 /*An adapted C++ implementation of RSA Data Securities MD5 algorithm.*/
 #include <kmdcodec.h>
 
-KNotesDataSource::KNotesDataSource(OSyncMember *m, OSyncHashTable *h)
-		:member(m), hashtable(h)
-{
-	connected = false;
-}
+#include <opensync/xmlformat/opensync_xmlformat.h>
+#include <opensync/xmlformat/opensync_xmlfield.h>
 
-bool KNotesDataSource::connect(OSyncContext *ctx)
+void KNotesDataSource::connect(OSyncPluginInfo *info, OSyncContext *ctx)
 {
-	osync_trace(TRACE_ENTRY, "%s(%p)", __func__, ctx);
-
+	osync_trace(TRACE_ENTRY, "%s(%p, %p)", __PRETTY_FUNCTION__, info, ctx);
+	
 	//connect to dcop
 	kn_dcop = KApplication::kApplication()->dcopClient();
 	if (!kn_dcop) {
 		osync_context_report_error(ctx, OSYNC_ERROR_INITIALIZATION, "Unable to make new dcop for knotes");
 		osync_trace(TRACE_EXIT_ERROR, "%s: Unable to make new dcop for knotes", __func__);
-		return FALSE;
+		return;
 	}
 
 	/*if (!kn_dcop->attach()) {
@@ -66,7 +61,7 @@ bool KNotesDataSource::connect(OSyncContext *ctx)
 	if (kn_dcop->isApplicationRegistered("kontact")) {
 		osync_context_report_error(ctx, OSYNC_ERROR_NO_CONNECTION, "Kontact is running. Please finish it");
 		osync_trace(TRACE_EXIT_ERROR, "%s: Kontact is running", __func__);
-		return false;
+		return;
 	}
 
 	//check knotes running
@@ -81,17 +76,12 @@ bool KNotesDataSource::connect(OSyncContext *ctx)
 
 	kn_iface = new KNotesIface_stub("knotes", "KNotesIface");
 
-	if (!osync_anchor_compare(member, "note", "true")) {
-		osync_trace(TRACE_INTERNAL, "Setting slow-sync for note");
-		osync_member_set_slow_sync(member, "note", TRUE);
-	}
-
-	connected = true;
-	osync_trace(TRACE_EXIT, "%s", __func__);
-	return true;
+	OSyncDataSource::connect(info, ctx);
+	
+	osync_trace(TRACE_EXIT, "%s", __PRETTY_FUNCTION__);
 }
 
-bool KNotesDataSource::disconnect(OSyncContext *ctx)
+void KNotesDataSource::disconnect(OSyncPluginInfo *, OSyncContext *ctx)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p)", __func__, ctx);
 
@@ -112,9 +102,8 @@ bool KNotesDataSource::disconnect(OSyncContext *ctx)
 	//delete kn_dcop;
 	//kn_dcop = NULL;
 
-	connected = false;
+	osync_context_report_success(ctx);
 	osync_trace(TRACE_EXIT, "%s", __func__);
-	return true;
 }
 
 static QString strip_html(QString input)
@@ -138,152 +127,175 @@ static QString strip_html(QString input)
 	return output.stripWhiteSpace();
 }
 
-bool KNotesDataSource::get_changeinfo(OSyncContext *ctx)
+void KNotesDataSource::get_changes(OSyncPluginInfo *info, OSyncContext *ctx)
 {
+#if 0
 	osync_trace(TRACE_ENTRY, "%s(%p)", __func__, ctx);
 	QMap <KNoteID_t,QString> fNotes;
 	//set Digest to rawResult
-	KMD5::Digest rawResult;
+	//KMD5::Digest rawResult;
 	KMD5 hash_value;
+	OSyncError *error = NULL;
 
 	fNotes = kn_iface->notes();
 	if (kn_iface->status() != DCOPStub::CallSucceeded) {
 		osync_context_report_error(ctx, OSYNC_ERROR_GENERIC, "Unable to get changed notes");
 		osync_trace(TRACE_EXIT_ERROR, "%s: Unable to get changed notes", __func__);
-		return FALSE;
+		return;
 	}
 
-	if (osync_member_get_slow_sync(member, "note")) {
-		osync_debug("kcal", 3, "Setting slow-sync for notes");
-		osync_hashtable_set_slow_sync(hashtable, "note");
+	if (osync_objtype_sink_get_slowsync(sink)) {
+		osync_trace(TRACE_INTERNAL, "Got slow-sync, resetting hashtable");
+		if (!osync_hashtable_slowsync(hashtable, &error)) {
+			osync_context_report_osyncerror(ctx, error);
+			osync_trace(TRACE_EXIT_ERROR, "%s: %s", __PRETTY_FUNCTION__, osync_error_print(&error));
+			return;
+		}
 	}
 
+	OSyncFormatEnv *formatenv = osync_plugin_info_get_format_env(info);
+	OSyncObjFormat *objformat = osync_format_env_find_objformat(formatenv, "xmlformat-note");
+
+	OSyncChange *chg = NULL;
+	OSyncData *odata = NULL;
+	OSyncChangeType changetype;
+
 	QMap<KNoteID_t,QString>::ConstIterator i;
 	for (i = fNotes.begin(); i != fNotes.end(); i++) {
 		/* XXX: don't report empty notes, knotes always
 		 * "keeps" at least one
 		 */
 		if (kn_iface->text(i.key()) == "") {
-			osync_debug("knotes", 4, "Skipping empty note");
+//			osync_debug("knotes", 4, "Skipping empty note");
 			continue;
 		}
 
-		osync_debug("knotes", 4, "Note key: %s", (const char*)i.key().local8Bit());
-		osync_debug("knotes", 4, "Note summary: %s", (const char*)i.data().local8Bit());
+//		osync_debug("knotes", 4, "Note key: %s", (const char*)i.key().local8Bit());
+//		osync_debug("knotes", 4, "Note summary: %s", (const char*)i.data().local8Bit());
 		osync_trace(TRACE_INTERNAL, "reporting notes %s\n", (const char*)i.key().local8Bit());
 
 		QString uid = i.key();
 		QString hash = NULL;
-		// Create osxml doc containing the note
-		xmlDoc *doc = xmlNewDoc((const xmlChar*)"1.0");
-		xmlNode *root = osxml_node_add_root(doc, "Note");
 
-		OSyncXMLEncoding enc;
-		enc.encoding = OSXML_8BIT;
-		enc.charset = OSXML_UTF8;
+		// Create XMLFormat-note
+		OSyncXMLFormat *xmlformat = osync_xmlformat_new("note", &error);
+		OSyncXMLField *xmlfield = osync_xmlfield_new(xmlformat, "Summary", &error);
 
-		// Set the right attributes
-		xmlNode *sum = xmlNewChild(root, NULL, (const xmlChar*)"", NULL);
 		QCString utf8str = i.data().utf8();
 		hash_value.update(utf8str);
-		osxml_node_set(sum, "Summary", utf8str, enc);
+		osync_xmlfield_set_key_value(xmlfield, "Content", utf8str);
 
 		utf8str = strip_html(kn_iface->text(i.key())).utf8();
 		hash_value.update(utf8str);
 		hash = hash_value.base64Digest ();
 		if (utf8str && !utf8str.isEmpty()) {
-			xmlNode *body = xmlNewChild(root, NULL, (const xmlChar*)"", NULL);
-			osxml_node_set(body, "Body", utf8str, enc);
+			xmlfield = osync_xmlfield_new(xmlformat, "Body", &error);
+			osync_xmlfield_set_key_value(xmlfield, "Content", utf8str);
 		}
 
 		// initialize the change object
-		OSyncChange *chg = osync_change_new();
+		chg = osync_change_new(&error);
+		if (!chg)
+			goto error;
 		osync_change_set_uid(chg, uid.local8Bit());
-		osync_change_set_member(chg, member);
 
-		// object type and format
-		osync_change_set_objtype_string(chg, "note");
-		osync_change_set_objformat_string(chg, "xml-note");
-		osync_change_set_data(chg, (char*)doc, sizeof(doc), 1);
+		odata = osync_data_new((char*)xmlformat, sizeof(OSyncXMLFormat *), objformat, &error);
+		if (!odata)
+			goto error;
 
-		osync_debug("knotes", 4, "Reporting note:\%s", osync_change_get_printable(chg));
+		osync_data_set_objtype(odata, osync_objtype_sink_get_name(sink));
+
+		//Now you can set the data for the object
+		osync_change_set_data(chg, odata);
+		osync_data_unref(odata);
+
+//		osync_debug("knotes", 4, "Reporting note:\%s", osync_change_get_printable(chg));
 
 		// Use the hash table to check if the object
 		// needs to be reported
 		osync_change_set_hash(chg, hash.data());
-		if (osync_hashtable_detect_change(hashtable, chg)) {
+
+		changetype = osync_hashtable_get_changetype(hashtable, chg);
+		osync_change_set_changetype(chg, changetype);
+
+		// Update change in hashtable ... otherwise it gets deleted!
+		osync_hashtable_update_change(hashtable, chg);
+		if (OSYNC_CHANGE_TYPE_UNMODIFIED != changetype)
 			osync_context_report_change(ctx, chg);
-			osync_hashtable_update_hash(hashtable, chg);
-		}
+
 		hash_value.reset();
 	}
 
-	osync_hashtable_report_deleted(hashtable, ctx, "note");
+	if (!report_deleted(info, ctx, objformat)) {
+		osync_trace(TRACE_EXIT_ERROR, "%s", __func__);
+		return;
+	}
 
+	osync_context_report_success(ctx);
 	osync_trace(TRACE_EXIT, "%s", __func__);
-	return true;
+	return;
+
+error:
+	if (chg)
+		osync_change_unref(chg);
+	osync_context_report_osyncerror(ctx, error);
+	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(&error));
+	osync_error_unref(&error);
+#endif
 }
 
-/** basic access method
- *
- * This method is used by both access() and commit_change() method,
- * so it shouldn't call osync_context_report_success(). On success,
- * it should just return true and let the caller report success() to
- * OpenSync
- */
-bool KNotesDataSource::__access(OSyncContext *ctx, OSyncChange *chg)
+void KNotesDataSource::commit(OSyncPluginInfo *, OSyncContext *ctx, OSyncChange *chg)
 {
+#if 0
 	osync_trace(TRACE_ENTRY, "%s(%p, %p)", __func__, ctx, chg);
 	OSyncChangeType type = osync_change_get_changetype(chg);
 
 	QString uid = osync_change_get_uid(chg);
 
 	//set Digest to rawResult
-	KMD5::Digest rawResult;
+	//KMD5::Digest rawResult;
 	KMD5 hash_value;
 
-	if (type != CHANGE_DELETED) {
+	if (type != OSYNC_CHANGE_TYPE_DELETED) {
 
-		// Get osxml data
-		xmlDoc *doc = (xmlDoc*)osync_change_get_data(chg);
+		// Get data
+		OSyncData *odata = osync_change_get_data(chg);
+		OSyncXMLFormat *xmlformat = (OSyncXMLFormat *)osync_data_get_data_ptr(odata);
 
-		xmlNode *root = xmlDocGetRootElement(doc);
-		if (!root) {
-			osync_context_report_error(ctx, OSYNC_ERROR_GENERIC, "Unable to get xml root element");
+		if (!xmlformat) {
+			osync_context_report_error(ctx, OSYNC_ERROR_GENERIC, "Unable to get xmlformat");
 			osync_trace(TRACE_EXIT_ERROR, "%s: Invalid data", __func__);
-			return false;
+			return;
 		}
 
-		if (xmlStrcmp((root)->name, (const xmlChar *) "Note")) {
-			osync_context_report_error(ctx, OSYNC_ERROR_GENERIC, "Wrong root element");
-			osync_trace(TRACE_EXIT_ERROR, "%s: Invalid data2", __func__);
-			return false;
+		if (strcmp("note", osync_xmlformat_get_objtype(xmlformat))) {
+			osync_context_report_error(ctx, OSYNC_ERROR_GENERIC, "Wrong xmlformat: %s", osync_xmlformat_get_objtype(xmlformat));
+			osync_trace(TRACE_EXIT_ERROR, "%s: Wrong xmlformat.", __func__);
+			return;
 		}
 
-		xmlChar *xmlStr;
-		char * tmpStr;
+		QString summary, body;
 
-		tmpStr = osync_change_get_printable(chg);
-		xmlStr = (xmlChar*) osxml_find_node(root, "Summary");
-		osync_trace(TRACE_INTERNAL, "Getting note %s and %s\n", tmpStr, xmlStr);
+	       OSyncXMLField *xmlfield = osync_xmlformat_get_first_field(xmlformat);
+		for (; xmlfield; xmlfield = osync_xmlfield_get_next(xmlfield)) {
+			osync_trace(TRACE_INTERNAL, "Field: %s", osync_xmlfield_get_name(xmlfield));
 
-		QString summary = (char*)xmlStr;
-		xmlFree(xmlStr);
-		g_free(tmpStr);
-
-		xmlStr = (xmlChar*) osxml_find_node(root, "Body");
-		QString body = (char*)xmlStr;
-		xmlFree(xmlStr);
+			if (!strcmp("Body", osync_xmlfield_get_name(xmlfield))) {
+				summary = osync_xmlfield_get_key_value(xmlfield, "Content");
+			} else if (!strcmp("Summary", osync_xmlfield_get_name(xmlfield))) {
+				body = osync_xmlfield_get_key_value(xmlfield, "Content");
+			}
+		}
 
 		QString hash;
 		switch (type) {
-			case CHANGE_ADDED: {
+			case OSYNC_CHANGE_TYPE_ADDED: {
 				osync_trace(TRACE_INTERNAL, "addding new \"%s\" and \"%s\"\n", (const char*)summary.local8Bit(), (const char*)body.local8Bit());
 				uid = kn_iface->newNote(summary, body);
 				if (kn_iface->status() != DCOPStub::CallSucceeded) {
 					osync_context_report_error(ctx, OSYNC_ERROR_GENERIC, "Unable to add new note");
 					osync_trace(TRACE_EXIT_ERROR, "%s: Unable to add new note", __func__);
-					return false;
+					return;
 				}
 
 				kn_iface->hideNote(uid);
@@ -296,19 +308,19 @@ bool KNotesDataSource::__access(OSyncContext *ctx, OSyncChange *chg)
 				osync_change_set_hash(chg, hash);
 				break;
 			}
-			case CHANGE_MODIFIED: {
+			case OSYNC_CHANGE_TYPE_MODIFIED: {
 				kn_iface->setName(uid, summary);
 				if (kn_iface->status() != DCOPStub::CallSucceeded) {
 					osync_context_report_error(ctx, OSYNC_ERROR_GENERIC, "Unable to set name");
 					osync_trace(TRACE_EXIT_ERROR, "%s: Unable to set name", __func__);
-					return false;
+					return;
 				}
 
 				kn_iface->setText(uid, body);
 				if (kn_iface->status() != DCOPStub::CallSucceeded) {
 					osync_context_report_error(ctx, OSYNC_ERROR_GENERIC, "Unable to set text");
 					osync_trace(TRACE_EXIT_ERROR, "%s: Unable to set text", __func__);
-					return false;
+					return;
 				}
 				hash_value.update(summary);
 				hash_value.update(body);
@@ -319,14 +331,14 @@ bool KNotesDataSource::__access(OSyncContext *ctx, OSyncChange *chg)
 			default: {
 				osync_context_report_error(ctx, OSYNC_ERROR_NOT_SUPPORTED, "Invalid change type");
 				osync_trace(TRACE_EXIT_ERROR, "%s: Invalid change type", __func__);
-				return false;
+				return;
 			}
 		}
 	} else {
 		system("dcop knotes KNotesIface hideAllNotes");
 		QString asdasd = "dcop knotes KNotesIface killNote " + uid + " true";
 		system((const char*)asdasd.local8Bit());
-		osync_debug("knotes", 4, "Deleting note %s", (const char*)uid.local8Bit());
+//		osync_debug("knotes", 4, "Deleting note %s", (const char*)uid.local8Bit());
 		/*kn_iface->killNote(uid, true);
 		if (kn_iface->status() != DCOPStub::CallSucceeded) {
 			osync_context_report_error(ctx, OSYNC_ERROR_GENERIC, "Unable to delete note");
@@ -335,31 +347,8 @@ bool KNotesDataSource::__access(OSyncContext *ctx, OSyncChange *chg)
 		}*/
 	}
 
-	osync_trace(TRACE_EXIT, "%s", __func__);
-	return true;
-}
-
-bool KNotesDataSource::commit_change(OSyncContext *ctx, OSyncChange *chg)
-{
-	osync_trace(TRACE_ENTRY, "%s(%p, %p)", __func__, ctx, chg);
-	if (!__access(ctx, chg)) {
-		osync_trace(TRACE_EXIT_ERROR, "%s: Unable to delete note", __func__);
-		return false;
-	}
-	osync_hashtable_update_hash(hashtable, chg);
+	osync_hashtable_update_change(hashtable, chg);
 	osync_context_report_success(ctx);
 	osync_trace(TRACE_EXIT, "%s", __func__);
-	return true;
-}
-
-bool KNotesDataSource::access(OSyncContext *ctx, OSyncChange *chg)
-{
-	osync_trace(TRACE_ENTRY, "%s(%p, %p)", __func__, ctx, chg);
-	if (!__access(ctx, chg)) {
-		osync_trace(TRACE_EXIT_ERROR, "%s: Unable to delete note", __func__);
-		return false;
-	}
-	osync_context_report_success(ctx);
-	osync_trace(TRACE_EXIT, "%s", __func__);
-	return true;
+#endif
 }
